import cythonpowered.random


ITERATIONS = 10000
FLOAT_INTERVAL = [0, 1]
INT_INTERVAL = [-100, 100]
UNIFORM_INTERVAL = [-123.456, 123.456]


def test_random():
    for i in range(ITERATIONS):
        result = cythonpowered.random.random()
        assert result >= FLOAT_INTERVAL[0]
        assert result <= FLOAT_INTERVAL[1]


def test_n_random():
    result = cythonpowered.random.n_random(ITERATIONS)
    assert len(result) == ITERATIONS
    assert len(set(result)) > 1

    for i in range(ITERATIONS):
        assert result[i] >= FLOAT_INTERVAL[0]
        assert result[i] <= FLOAT_INTERVAL[1]


def test_randint():
    for i in range(ITERATIONS):
        result = cythonpowered.random.randint(*INT_INTERVAL)
        assert result >= INT_INTERVAL[0]
        assert result <= INT_INTERVAL[1]


def test_n_randint():
    result = cythonpowered.random.n_randint(*INT_INTERVAL, ITERATIONS)
    assert len(result) == ITERATIONS
    assert len(set(result)) > 1

    for i in range(ITERATIONS):
        assert result[i] >= INT_INTERVAL[0]
        assert result[i] <= INT_INTERVAL[1]


def test_uniform():
    for i in range(ITERATIONS):
        result = cythonpowered.random.uniform(*UNIFORM_INTERVAL)
        assert result >= UNIFORM_INTERVAL[0]
        assert result <= UNIFORM_INTERVAL[1]


def test_n_uniform():
    result = cythonpowered.random.n_uniform(*UNIFORM_INTERVAL, ITERATIONS)
    assert len(result) == ITERATIONS
    assert len(set(result)) > 1

    for i in range(ITERATIONS):
        assert result[i] >= UNIFORM_INTERVAL[0]
        assert result[i] <= UNIFORM_INTERVAL[1]


def test_choice():
    population = cythonpowered.random.n_randint(*INT_INTERVAL, ITERATIONS)
    result = cythonpowered.random.choice(population)
    assert result in population


def test_choices():
    population = cythonpowered.random.n_randint(*INT_INTERVAL, ITERATIONS)
    sample_size = ITERATIONS // 2
    result = cythonpowered.random.choices(population, sample_size)

    assert len(result) == sample_size
    assert len(set(result)) > 1
    assert set(result).issubset(set(population))



from scripts.benchmark._base import (
    BaseFunctionDefinition,
    BaseFunctionBenchmark,
    BaseModuleBenchmark,
)

import random as py_random
import cythonpowered.random as cy_random


# =============================================================================
class PythonRandomRandomDef(BaseFunctionDefinition):
    function = py_random.random
    reference = "random.random"


class CythonRandomRandomDef(BaseFunctionDefinition):
    function = cy_random.random
    reference = "cythonpowered.random.random"


class CythonRandomNRandomDef(BaseFunctionDefinition):
    function = cy_random.n_random
    reference = "cythonpowered.random.n_random"


class RandomBenchmarkDefinition(BaseFunctionBenchmark):
    python_function = PythonRandomRandomDef
    cython_function = CythonRandomRandomDef
    cython_n_function = CythonRandomNRandomDef


# =============================================================================
class PythonRandomRandintDef(BaseFunctionDefinition):
    function = py_random.randint
    reference = "random.randint"


class CythonRandomRandintDef(BaseFunctionDefinition):
    function = cy_random.randint
    reference = "cythonpowered.random.randint"


class CythonRandomNRandintDef(BaseFunctionDefinition):
    function = cy_random.n_randint
    reference = "cythonpowered.random.n_randint"


class RandintBenchmarkDefinition(BaseFunctionBenchmark):
    python_function = PythonRandomRandintDef
    cython_function = CythonRandomRandintDef
    cython_n_function = CythonRandomNRandintDef
    args = [-1000000, 1000000]


# =============================================================================
class PythonRandomUniformDef(BaseFunctionDefinition):
    function = py_random.uniform
    reference = "random.uniform"


class CythonRandomUniformDef(BaseFunctionDefinition):
    function = cy_random.uniform
    reference = "cythonpowered.random.uniform"


class CythonRandomNUniformDef(BaseFunctionDefinition):
    function = cy_random.n_uniform
    reference = "cythonpowered.random.n_uniform"


class UniformBenchmarkDefinition(BaseFunctionBenchmark):
    python_function = PythonRandomUniformDef
    cython_function = CythonRandomUniformDef
    cython_n_function = CythonRandomNUniformDef
    args = [-123456.789, 123456.789]


# =============================================================================
class PythonRandomChoiceDef(BaseFunctionDefinition):
    function = py_random.choice
    reference = "random.choice"


class CythonRandomChoiceDef(BaseFunctionDefinition):
    function = cy_random.choice
    reference = "cythonpowered.random.choice"


class ChoiceBenchmarkDefinition(BaseFunctionBenchmark):
    python_function = PythonRandomChoiceDef
    cython_function = CythonRandomChoiceDef
    args = [cy_random.n_randint(-100000, 100000, 10000)]


# =============================================================================
class PythonRandomChoicesDef(BaseFunctionDefinition):
    function = py_random.choices
    reference = "random.choices"


class CythonRandomChoicesDef(BaseFunctionDefinition):
    function = cy_random.choices
    reference = "cythonpowered.random.choices"


class ChoicesBenchmarkDefinition(BaseFunctionBenchmark):
    python_function = PythonRandomChoicesDef
    cython_function = CythonRandomChoicesDef
    args = [cy_random.n_randint(-100000, 100000, 10000)]
    kwargs = {"k": 100}
    runs = [1000, 10000, 100000]


# =============================================================================
class RandomBenchmark(BaseModuleBenchmark):
    MODULE = "random"
    BENCHMARKS = [
        RandomBenchmarkDefinition,
        RandintBenchmarkDefinition,
        UniformBenchmarkDefinition,
        ChoiceBenchmarkDefinition,
        ChoicesBenchmarkDefinition,
    ]



import argparse
import sys

from scripts.benchmark.benchmark_runner import BenchmarkRunner
from cythonpowered import VERSION


def run_benchmark():
    BenchmarkRunner()
    sys.exit(0)


def print_version():
    print(VERSION)
    sys.exit(0)


def main():
    parser = argparse.ArgumentParser(
        prog="cythonpowered",
        description="Utilities for the cythonpowered Python library",
    )

    parser.add_argument("-b", "--benchmark", help="run benchmark", action="store_true")
    parser.add_argument("-v", "--version", help="print version", action="store_true")
    args = parser.parse_args()

    passed_args = [
        arg for arg in args.__dict__ if args.__dict__[arg] not in [False, None]
    ]
    if len(passed_args) == 0:
        parser.print_help()

    if args.benchmark is True:
        run_benchmark()

    if args.version is True:
        print_version()


if __name__ == "__main__":
    main()



import os
from setuptools import Extension, setup
import subprocess
import sys
from cythonpowered import VERSION, MODULES as CYTHON_MODULES


NAME = "cythonpowered"
LICENSE = "GNU GPLv3"
DESCRIPTION = "Cython-powered replacements for popular Python functions. And more."
AUTHOR = "Lucian Croitoru"
AUTHOR_EMAIL = "lucianalexandru.croitoru@gmail.com"
URL = "https://github.com/lucian-croitoru/cythonpowered"

KEYWORDS = ["python", "cython", "random", "performance"]
CLASSIFIERS = [
    "Development Status :: 2 - Pre-Alpha",
    "Intended Audience :: Developers",
    "Programming Language :: Python :: 3",
    "Operating System :: Unix",
]
SETUP_REQUIRES = ["Cython>=3.0.0", "setuptools>=74.0.0"]
INSTALL_REQUIRES = ["psutil>=6.0.0", "py-cpuinfo>=9.0.0"]
PYTHON_MODULES = [NAME, "scripts", "scripts.benchmark"]

install_cython = subprocess.Popen(["pip", "install"] + SETUP_REQUIRES)
install_cython.wait()

from Cython.Build import cythonize

# Get long_description from README
with open("README.md", "r") as f:
    long_description = f.read()

# Get CHANGELOG
with open("CHANGELOG.md", "r") as f:
    changelog = f.read()

long_description = long_description + "\n\n" + changelog


# Get Cython module information
cython_file_list = [
    {
        "module_name": f"{NAME}.{module}.{module}",
        "module_source": [
            os.path.join(NAME, module, "*.pyx"),
        ],
    }
    for module in CYTHON_MODULES
]


# Build Cython extensions
cython_module_list = []

for f in cython_file_list:
    extension = Extension(
        name=f["module_name"],
        sources=f["module_source"],
        language="c",
        extra_compile_args=["-fopenmp"],
        extra_link_args=["-fopenmp"],
        define_macros=[("NPY_NO_DEPRECATED_API", "NPY_1_7_API_VERSION")],
    )
    cython_module_list.append(extension)


# Set build_ext --inplace argument explicitly
sys.argv = sys.argv + ["build_ext", "--inplace"]

setup(
    name=NAME,
    version=VERSION,
    license=LICENSE,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    packages=PYTHON_MODULES + [f"{NAME}.{module}" for module in CYTHON_MODULES],
    keywords=KEYWORDS,
    classifiers=CLASSIFIERS,
    setup_requires=SETUP_REQUIRES,
    install_requires=SETUP_REQUIRES + INSTALL_REQUIRES,
    scripts=[],
    ext_modules=cythonize(module_list=cython_module_list, language_level="3"),
    package_data={"": ["*.pyx"]},
    include_package_data=True,
    entry_points={
        "console_scripts": ["cythonpowered=scripts.main:main"],
    },
)
